/*creates daily VIC forcing files using historic gridded data and ctl files
  produced by the GCM downscaling procedure */
/*Assumes gridded observed daily files start on Jan 1 of designated year */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#define LEAPYR(y) (!((y)%400)) || (!((y%4) && ((y)%100)))
#define MAXYEARS 100 /*max years in input data record */

float ***get_3d_mem_float(int dim1, int dim2, int dim3);

int main(int argc, char *argv[]) 
{
  
  FILE *fp,*fpout,*fpctl;
  char infile[200],outf[200],ctlf[200];
  unsigned short int prec;
  short int t1, t2, w;
  int x,lines,i;
  int year,st_year,ndys,mn,dy;
  float ***prcp,***tmax,***tmin,***wind;
  int nd[] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
  int ctlmn,ctlyr,ctlndys;
  float ps,tf;
  float mult[] = {40.0, 100.0, 100.0, 100.0};
  
  if(argc!=6) {
    printf("Usage: %s <daily infile> <control ctl file> <outdirectory> <startyear> <startmo> <startday> <output startyr> <output endyr> \n",argv[0]);
    printf("\tThis routine scales vicinput binary data using ctl files\n");
    exit(0);
  }

  prcp = get_3d_mem_float(MAXYEARS,13,32);
  tmax = get_3d_mem_float(MAXYEARS,13,32);
  tmin = get_3d_mem_float(MAXYEARS,13,32);
  wind = get_3d_mem_float(MAXYEARS,13,32);

  strcpy(infile,argv[1]);
  strcpy(ctlf,argv[2]);
  strcpy(outf,argv[3]);
  lines = atoi(argv[4]);  /* starting date for data */
  st_year = atoi(argv[5]);    /* starting date for data */
  
  if((fp = fopen(infile, "rb")) == NULL) {
    printf("Cannot open infile %s\n", infile);
    exit(1);
  }
  
  year = st_year;
  mn=1;
  dy=0;
  
  /*must start on 1/1 of st_year*/
  for (x=0;x<lines;x++) {
    
    dy++;
    
    if(fread(&prec,sizeof(unsigned short int),1,fp) != 1)
      printf("read prec err rec %d %s\n", x,infile);
    if(fread(&t1,sizeof(short int),1,fp) != 1)
      printf("read t1 err rec %d %s\n", x,infile);
    if(fread(&t2,sizeof(short int),1,fp) != 1)
      printf("read t2 err rec %d %s\n", x,infile);
    if(fread(&w,sizeof(short int),1,fp) != 1)
      printf("read t3 err rec %d %s\n", x,infile);
    
    prcp[year-st_year][mn][dy] = (float) (prec)/mult[0];
    tmax[year-st_year][mn][dy] = (float) (t1)/mult[1];
    tmin[year-st_year][mn][dy] = (float) (t2)/mult[2];
    wind[year-st_year][mn][dy] = (float) (w)/mult[3];
    
    ndys = nd[mn];
    
    if((LEAPYR(year)) && mn==2) {ndys=29;}

    /* for resampling, add 2/29 to all years in case it is used in a leap year */
    if((! LEAPYR(year)) && mn==2 && dy==28) {
      prcp[year-st_year][mn][dy+1] = (float) (prec)/mult[0];
      tmax[year-st_year][mn][dy+1] = (float) (t1)/mult[1];
      tmin[year-st_year][mn][dy+1] = (float) (t2)/mult[2];
      wind[year-st_year][mn][dy+1] = (float) (w)/mult[3];
    }

    if(dy>=ndys) {
      mn++;
      dy=0;
      if(mn>12) {
	mn=1;
	year++;
      }
    }
    
  }
  fclose(fp);
  
  /*now go through ctl file and find appropriate days, and scale them */
  
  if((fpout = fopen(outf, "wb")) == NULL) {
    printf("Cannot open file %s\n", outf);
    exit(1);
  }
  if((fpctl = fopen(ctlf, "r")) == NULL) {
    printf("Cannot open file %s\n", ctlf);
    exit(1);
  }
  
  while(fscanf(fpctl,"%d %d %d %f %f",&ctlmn,&ctlyr,&ctlndys,&ps,&tf)!=EOF) { 
    
    for(i=1;i<=ctlndys;i++) {
      
      /* convert back to multiplied ints  -- same format as original data */
      prec = (unsigned short int) ((prcp[ctlyr-st_year][ctlmn][i]*ps)*mult[0]);
      t1 = (short int) ((tmax[ctlyr-st_year][ctlmn][i]+tf)*mult[1]);
      t2 = (short int) ((tmin[ctlyr-st_year][ctlmn][i]+tf)*mult[2]);
      w = (short int) (wind[ctlyr-st_year][ctlmn][i]*mult[3]);
      
      fwrite(&prec,sizeof(unsigned short int),1,fpout);
      fwrite(&t1,sizeof(short int),1,fpout);
      fwrite(&t2,sizeof(short int),1,fpout);
      fwrite(&w,sizeof(short int),1,fpout);
    }
  }
  
  fclose(fpout);
  fclose (fpctl);
}

/******************************************************************/
float ***get_3d_mem_float(int dim1, int dim2, int dim3)
{
  float ***array_3d;
  int k,j;

  if(!(array_3d = (float***) calloc(dim1,sizeof(float**)))) {
    printf("Cannot allocate memory in get_3d_mem_float\n");
    exit(1); }
  for(k=0; k<dim1;k++) {
    if(!(array_3d[k] = (float**) calloc(dim2,sizeof(float*)))) {
      printf("Cannot allocate memory in get_3d_mem_float\n");
      exit(1); }
    for(j=0; j<dim2;j++) {
      if(!(array_3d[k][j] = (float*) calloc(dim3,sizeof(float)))) {
        printf("Cannot allocate memory in get_3d_mem_float\n");
        exit(1); }
    }
  }
  return array_3d;
}
